unit DIBEditor;

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: DIBEditor.PAS, released August 28, 2000.

The Initial Developer of the Original Code is Peter Morris (pete@stuckindoors.com),
Portions created by Peter Morris are Copyright (C) 2000 Peter Morris.
All Rights Reserved.

Purpose of file:
Editor for DIBs, load / save / import mask etc.

Contributor(s):
None as yet


Last Modified: August 31, 2000

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org
or at http://www.stuckindoors.com/dib


Known Issues:
Add an Eyedropper for chosing transparent colour.


Date : 31 - Aug, 2000 :
By :pete@stuckindoors.com
Changes
Made TxxxxxxEditor to TxxxxxxxProperty to comply with VCL standards.
Made sure all unit names start with DIB to avoice conflicts with other people's
component packs.

-----------------------------------------------------------------------------}


interface

{$i OpenSource/dfs.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  cDIBControl, cDIBImage, StdCtrls, cDIBPanel, ExtCtrls, Menus, ExtDlgs,
  {$IFDEF DFS_NO_DSGNINTF}
  DesignEditors, DesignIntf,
  {$ELSE}
  DsgnIntf,
  {$ENDIF}
  Spin, jpeg, ComCtrls, TypInfo, cDIB, cDIBImageList, ImgList, ToolWin,
  Buttons;

type
  TAbstractSuperDIBProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes : TPropertyAttributes; override;
  end;

  TfmDIBEditor = class(TForm)
    pnlControl: TPanel;
    opd1: TOpenPictureDialog;
    cdColor: TColorDialog;
    DIBImageList1: TDIBImageList;
    dicRender: TDIBImageContainer;
    DIBImage1: TDIBImage;
    stbStatus: TStatusBar;
    ImageList1: TImageList;
    gbTransparency: TGroupBox;
    Label4: TLabel;
    shTransparentColor: TShape;
    cbTransparentMode: TComboBox;
    cbMasked: TCheckBox;
    Panel1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label5: TLabel;
    seAngle: TSpinEdit;
    pbAngle: TProgressBar;
    seOpacity: TSpinEdit;
    pbOpacity: TProgressBar;
    seScale: TSpinEdit;
    pbScale: TProgressBar;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Panel2: TPanel;
    btnImageImport: TSpeedButton;
    btnMaskImport: TSpeedButton;
    btnOK: TSpeedButton;
    procedure seAngleChange(Sender: TObject);
    procedure seOpacityChange(Sender: TObject);
    procedure cbMaskedClick(Sender: TObject);
    procedure seScaleChange(Sender: TObject);
    procedure pbAngleMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure pbAngleMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pbAngleMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pbOpacityMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure pbScaleMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure cbTransparentModeChange(Sender: TObject);
    procedure shTransparentColorMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btnImageImportClick(Sender: TObject);
    procedure btnMaskImportClick(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FCaptureMouse   : Boolean;
    procedure ClipCurs(AComponent : TControl);
    procedure UpdateStatusBar;
  public
    { Public declarations }
    TheImage        : TMemoryDIB;
  end;

implementation

uses DIBImages;

{$R *.DFM}

{ TAbstractSuperDIBProperty }

procedure TAbstractSuperDIBProperty.Edit;
var
  EdForm            : TfmDIBEditor;
begin
  EdForm := TfmDIBEditor.Create(Application);
  with EdForm do
  try
    TheImage := DIBImageList1.DIBImages[0].DIB;
    with TheImage do begin
      Assign(TAbstractSuperDIB(GetOrdValue));
      seAngle.Value := Angle;
      pbAngle.Position := Angle;
      seOpacity.Value := Opacity;
      pbOpacity.Position := Opacity;
      seScale.Value := Round(Scale);
      pbScale.Position := seScale.Value;
      cbMasked.Checked := Masked;
      // CHANGED AZZA
      if (not Transparent) then
        cbTransparentMode.ItemIndex:=0
      else
        cbTransparentMode.ItemIndex:=1+Ord(TransparentMode);
      shTransparentColor.Brush.Color := TransparentColor;
    end;

    ShowModal;

    if ModalResult = mrOK then with TAbstractSuperDIB(GetOrdValue) do begin
      Assign(TheImage);
      Self.Designer.Modified;
    end;
  finally
    EdForm.Free;
  end;
end;

function TAbstractSuperDIBProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paReadOnly, paDialog];
end;

{ TfmDIBEditor }

// AZZA
procedure TfmDIBEditor.UpdateStatusBar;
begin
  stbStatus.SimpleText:=DIBImageList1.DIBImages[0].DisplayName+' ('+IntToStr(TheImage.Width)+' x '+IntToStr(TheImage.Height)+')';
end;

procedure TfmDIBEditor.FormCreate(Sender: TObject);
begin
  FCaptureMouse := False;
  TheImage := DIBImageList1.DIBImages[0].DIB;
  DIBImage1.DIBImageList := DIBImageList1;
  DIBImage1.IndexMain.DIBIndex :=0;

  // AZZA
  UpdateStatusBar;
end;

// AZZA
procedure TfmDIBEditor.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if (Ord(Key)=VK_ESCAPE) then
    Close;
end;

procedure TfmDIBEditor.seAngleChange(Sender: TObject);
begin
  TheImage.Angle := seAngle.Value;
  DIBImage1.Angle := seAngle.Value;
  pbAngle.Position := seAngle.Value;
end;

procedure TfmDIBEditor.seOpacityChange(Sender: TObject);
begin
  TheImage.Opacity := seOpacity.Value;
  DIBImage1.Opacity := seOpacity.Value;
  pbOpacity.Position := seOpacity.Value;
end;

procedure TfmDIBEditor.cbMaskedClick(Sender: TObject);
begin
  TheImage.Masked := cbMasked.Checked;
end;

procedure TfmDIBEditor.seScaleChange(Sender: TObject);
begin
  TheImage.Scale := seScale.Value;
  DIBImage1.Scale := seScale.Value;
  pbScale.Position := seScale.Value;
end;

procedure TfmDIBEditor.pbAngleMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if FCaptureMouse then seAngle.Value := Trunc(X / pbAngle.Width * 359);
end;

procedure TfmDIBEditor.pbAngleMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FCaptureMouse := True;
  ClipCurs(TControl(Sender));
end;

procedure TfmDIBEditor.ClipCurs(AComponent: TControl);
var
  R                 : TRect;
begin
  R := AComponent.ClientRect;
  R.TopLeft := AComponent.ClientToScreen(R.TopLeft);
  R.Bottom := R.Bottom + R.Top;
  R.Right := R.Right + R.Left;
  ClipCursor(@R);
end;

procedure TfmDIBEditor.pbAngleMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FCaptureMouse := False;
  ClipCursor(nil);
end;

procedure TfmDIBEditor.pbOpacityMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if FCaptureMouse then seOpacity.Value := Trunc(X / pbOpacity.Width * 255);
end;

procedure TfmDIBEditor.pbScaleMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  if FCaptureMouse then seScale.Value := Trunc(X / pbScale.Width * pbScale.Max)+1;
end;

procedure TfmDIBEditor.cbTransparentModeChange(Sender: TObject);
begin
  // CHANGED AZZA
  if cbTransparentMode.ItemIndex=0 then
    TheImage.Transparent:=False
  else begin
    TheImage.Transparent:=True;
    TheImage.TransparentMode:=TTransparentMode(cbTransparentMode.ItemIndex-1);
    shTransparentColor.Brush.Color:=TheImage.TransparentColor;
  end;
end;

procedure TfmDIBEditor.shTransparentColorMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if cdColor.Execute then begin
    shTransparentColor.Brush.Color := cdColor.Color;
    TheImage.TransparentColor := cdColor.Color;
  end;
end;

procedure TfmDIBEditor.btnImageImportClick(Sender: TObject);
begin
  if opd1.execute then
    TheImage.ImportPicture(opd1.Filename);

  // AZZA
  UpdateStatusBar;
end;

procedure TfmDIBEditor.btnMaskImportClick(Sender: TObject);
begin
  if opd1.Execute then
    TheImage.ImportMask(opd1.Filename);
end;

procedure TfmDIBEditor.btnOkClick(Sender: TObject);
begin
  ModalResult:=mrOk;
end;

end.
